package ConcurrentFramework;

import java.sql.Timestamp;

public class Philosopher implements Runnable {

    private final Object leftFork;
    private final Object rightFork;

    Philosopher(Object left, Object right) {
        this.leftFork = left;
        this.rightFork = right;
    }

    private void doAction(String action) throws InterruptedException {
    	System.out.printf("%25s > %s: %s\n", new Timestamp(System.currentTimeMillis())
    			                            , Thread.currentThread().getName()
    			                            , action);
        Thread.sleep(((int) (Math.random() * 100)));
    }

    @Override
    public void run() {
    	
        try {
            while (true) {
            	
                doAction("Thinking."); // Thinking.
                synchronized (leftFork) {
                	
                    doAction("Picked up left fork.");
                    
                    synchronized (rightFork) {
                    	
                        doAction("Picked up right fork; eating."); // Eating.
                        doAction("Put down right fork.");
                        
                    }
                    
                    doAction("Put down left fork; returning to thinking.");
                }
                
            }
        } catch (InterruptedException e) {
            Thread.currentThread().interrupt();
        }
        
    }
}