package ConcurrentFramework;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.List;
import java.util.concurrent.ForkJoinTask;
import java.util.concurrent.RecursiveTask;

public class RecursiveTaskDemo {
	
	@SuppressWarnings("serial")
	static class CustomRecursiveTask extends RecursiveTask<Integer> {
		
	    private int[] arr;
	    private int M = 2;
	 
	    private static final int threshold = 1000;
	 
	    public CustomRecursiveTask(int[] arr) {
	        this.arr = arr;
	    }
	 
	    @Override
	    protected Integer compute() {
	    	
	        if (arr.length > threshold) {
	            return ForkJoinTask.invokeAll(createSubtasks())
	                               .stream()
	                               .mapToInt(ForkJoinTask::join)
	                               .sum();
	        } else {
	            return processing(arr);
	        }
	        
	    }
	 
	    private Collection<CustomRecursiveTask> createSubtasks() {
	    	
	    	int unit_size = arr.length / M;
	    	
	        List<CustomRecursiveTask> dividedTasks = new ArrayList<>();
	        
	        int start = 0;
	        while (start + unit_size < arr.length) {
	        	dividedTasks.add(new CustomRecursiveTask(Arrays.copyOfRange(arr, start, start + unit_size)));
	        	start += unit_size;
	        }
	        dividedTasks.add(new CustomRecursiveTask(Arrays.copyOfRange(arr, start, arr.length)));
	        	        
	        return dividedTasks;
	        
	    }
	 
	    private Integer processing(int[] arr) {
	        return Arrays.stream(arr)
                         .filter(x -> x > 0)
                         .sum();
	    }
	}

	public static void main(String[] args) {

		int N = 100000000;
		int[] data = new int[N];
		for (int i = 0; i < N; i++ ) {
			data[i] = (int) (Math.random() * N) - N / 2;
		}
		
		System.out.println(new CustomRecursiveTask(data).compute());
		
		int result = Arrays.stream(data)
                           .filter(x -> x > 0)
                           .sum();
		System.out.println(result);
		
	}

}
